﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace CarFire
{
    /// <summary>
    /// A key entity.  Keys can be used to unlock doors... what a surprise.
    /// </summary>
    public class Key : Trigger
    {
        #region Public Methods

        /// <summary>
        /// Construct a key entity.
        /// </summary>
        /// <param name="identifier">The entity identifier.</param>
        /// <param name="position">The position.</param>
        /// <param name="info">The key-value pairs.</param>
        /// <param name="game">The game reference.</param>
        public Key(char identifier, Point position, Dictionary<string, string> info, Game game) :
            base(identifier, position, info, game)
        {
            mPosition = new Vector2(position.X, position.Y);
            mGame = game;
        }

        public override void LoadContent(ContentManager contentManager)
        {
            mTexture = contentManager.Load<Texture2D>("default");
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            Rectangle position = mGame.State.Map.GetRectangleFromCoordinates(mPosition);
            spriteBatch.Draw(mTexture, position, Color.White);
        }

        #endregion


        #region Private Variables

        Texture2D mTexture;
        Game mGame;
        Vector2 mPosition;

        #endregion
    }
}
